function [w lambda k] = glopridu2_algorithm(X,Y,blocks,tau,w0,lambda0,sigma,stop_par)
% GLOPRIDU2_ALGORITHM argmin of least squares error with GLO penalty, using
% proximal methods.
% 
% [W] = GLOPRIDU2_ALGORITHM(X,Y,BLOCKS,TAU) returns the solution of GLO
%   regularization with l1 parameter TAU.
%   X is the input data matrix (nxd), Y is the Nx1 label vector, BLOCKS is 
%   a cell array, where the g-th element contains the indexes of the 
%   features in block g.
% 
% [W] = GLOPRIDU2_ALGORITHM(X,Y,BLOCKS,TAU,W0) uses W0 as 
%   initialization vector for W
% 
% [W] = GLOPRIDU2_ALGORITHM(X,Y,BLOCKS,TAU,W0,SIGMA) uses SIGMA as step size
% 
% [W] = GLOPRIDU2_ALGORITHM(X,Y,BLOCKS,TAU,W0,SIGMA,STOP_PAR) stops
%   according to the fileds MAX_ITER_EXT, MAX_ITER_INT, TOL_EXT, TOL_INT in
%   STOP_PAR. If some of these fields are missing, then uses default
%   values.
% 
% [W,LAMBDA] = GLOPRIDU2_ALGORITHM(...) also returns the last value of the dual 
%    variable lambda used in the inner projection.
% 
% [W,LAMBDA,K] = GLOPRIDU2_ALGORITHM(...) also returns the number of iterations

   
    if nargin<4; error('too few inputs!'); end
    if nargin<5; w0 = []; end
    if nargin<6; lambda0 = []; end
    if nargin<7; sigma=[]; end 
    if nargin<8; stop_par = struct(); end
    if nargin>8; error('too many inputs!'); end

    if isfield(stop_par,'max_iter_ext');
        max_iter_ext = stop_par.max_iter_ext;
    else
        max_iter_ext = 1e4;
    end
    if isfield(stop_par,'max_iter_int');
        max_iter_int = stop_par.max_iter_int;
    else
        max_iter_int = 1e2;
    end
    if isfield(stop_par,'tol_ext');
        tol_ext = stop_par.tol_ext;
    else
        tol_ext = 1e-6;
    end
    if isfield(stop_par,'tol_int');
        tol_int = stop_par.tol_int;
    else
        tol_int = 1e-4;
    end
    clear stop_par;
    
    [n,d] =size(X);
    XT = X';
    
    % if sigma is not specified in input, evaluates it as (a+b)/(n*2)
    if isempty(sigma);
        sigma = normest(X)^2/n; %step size
    end

    XT = XT./(n*sigma);
    
    % initialization 
    if isempty(w0); w0 = zeros(d,1); end
    k=0;
    stop = 0;
    w = w0;
    h = w0;
    t = .5;
    c = 1;
    lambda_prev = lambda0;

    % GULER's iterative scheme, based on proximal method
%     fprintf('\nq_int = ')
    while or(and(k<max_iter_ext,~stop),k<2) 
        
        k = k+1;
        w_prev = w;   
        tp = t;
        
        t = (-c +sqrt(c^2+8*c))/4;
        fact1 = (1-t+t/tp);
        fact2 = ((tp-1)*(t/tp));

        % compute the proximity operator with tolerance depending on k
        [w,q,lambda] = glo2_prox(h + XT*(Y-X*h),tau/sigma,blocks,lambda_prev,tol_int*k^(-3/2),max_iter_int);
%         fprintf('%d,',q)
        h = w.*fact1 + w_prev.*fact2;
        lambda_prev = lambda; %warm starting
        
        stop = norm(w-w_prev)<=norm(w_prev)*tol_ext;
    end
end
