function [pred] = glopridu_pred(model,Xtest,Ytest,err_type)
%Predicts labels on test set
%   [PRED] = GLOPRIDU_PRED(MODEL,XTEST) Given MODEL from GLOPRIDU_KCV predicts labels
%   for test set Xtest. 
%   PRED's fields:
%       -Y_1STEP(if MODEL contains field BETA_1STEP)
%       -Y_2STEPS(if MODEL contains field BETA_2STEPS)
%   [PRED] = GLOPRIDU_PRED(MODEL,XTEST,YTEST) Given MODEL from 
%   GLOPRIDU_KCV predicts labels and computes means square errror for test set XTEST. 
%   PRED's fields:
%       if MODEL contains field BETA_1STEP
%           -Y_1STEP: estimated labels
%           -ERR_1STEP: mean square error
%       if MODEL contains field BETA_2STEPS
%           -Y_2STEPS:estimated labels
%           -ERR_2STEPS: mean square error 
%   [PRED] = GLOPRIDU_PRED(MODEL,XTEST,YTEST,ERR_TYPE) if ERR_TYPe='regr' 
%       compute means square errror for test set XTEST. If ERR_TYPE='class'
%       computes classification error. If ERR_TYPE = [w_pos, w_neg],
%       computes weighted classification error.
% 
%   See also GLOPRIDU_KCV

if nargin==3, err_type = 'regr'; end

ntest = size(Xtest,1);
if isfield(model,'meanX');
    for i = 1:ntest;
        Xtest(i,:) = (Xtest(i,:)-model.meanX);   
    end
end
if isfield(model,'stdevsX');
    for i = 1:ntest;
        Xtest(i,:) = Xtest(i,:)./model.stdevsX;   
    end
end
if ~isfield(model,'meanY');
    model.meanY = 0;
end

if isfield(model,'beta_1step')
    pred.y_1step = Xtest*model.beta_1step + model.meanY;
end
if isfield(model,'beta_2steps')
    pred.y_2steps = Xtest*model.beta_2steps + model.meanY;
end


if nargin>2;    
    if isequal(err_type,'regr');
        if isfield(model,'beta_1step')    
            pred.err_1step = norm(pred.y_1step-(Ytest))^2/length(Ytest);
        end
        if isfield(model,'beta_2steps')    
            pred.err_2steps = norm(pred.y_2steps-(Ytest))^2/length(Ytest);
        end
    else
        npos = sum(Ytest>0);
        nneg = sum(Ytest<0);
        if strcmp(err_type,'class'); 
            err_type = [npos/(npos+nneg) nneg/(npos+nneg)]; 
        end
        class_fraction = err_type;    
        if isfield(model,'beta_1step')
            pred.y_1step = sign(pred.y_1step);
            pred.err_1step = 0;
            if npos>0;
                err_pos = sum((pred.y_1step(Ytest>0)~=sign(Ytest(Ytest>0))))/npos;
                pred.err_1step = pred.err_1step + err_pos*max(class_fraction(1),nneg==0);
            end
            if nneg>0;
                err_neg = sum((pred.y_1step(Ytest<0)~=sign(Ytest(Ytest<0))))/nneg;
                pred.err_1step = pred.err_1step + err_neg*max(class_fraction(2),npos==0);
            end
        end
        
        if isfield(model,'beta_2steps')
            pred.y_2steps = sign(pred.y_2steps);
            pred.err_2steps = 0;
            if npos>0;
                err_pos = sum((pred.y_2steps(Ytest>0)~=sign(Ytest(Ytest>0))))/npos;
                pred.err_2steps = pred.err_2steps + err_pos*max(class_fraction(1),nneg==0);
            end
            if nneg>0;
                err_neg = sum((pred.y_2steps(Ytest<0)~=sign(Ytest(Ytest<0))))/nneg;
                pred.err_2steps = pred.err_2steps + err_neg*max(class_fraction(2),npos==0);
            end
        end
    end
end