function [beta k] = grl_algorithm(X,Y,blocks,tau,smooth_par,beta0,sigma0,max_iter, tol)
% GRL_ALGORITHM Returns the minimizer of the empirical error penalized
% with l1 or grl penalties FISTA.
% 
%   [BETA] = GRL_ALGORITHM(X,Y,BLOCKS,TAU) returns the solution of the
%   group-lasso algorithm with sparsity parameter TAU (smoothness parameter 0).
%   BLOCKS is either the number of blocks (with equal cardinality), or a 
%   cell array, where element i contains the indexes of the features in 
%   block i.
%   If the input data X is a NxD matrix, and the labels Y are a Nx1 vector,
%   BETA is a Dx1 vector. 
%   The step size is A/N, where A is the  largest eigenvalue of X'*X and N is the number of training samples.
%   The algorithm stops when value of the regularized empirical error reaches convergence.
% 
%   [BETA] = GRL_ALGORITHM(X,Y,BLOCKS,TAU,SMOOTH_PAR) returns the solution of the grl
%   regularization with sparsity parameter TAU and smoothness parameter
%   A/N*SMOOTH_PAR. The step size is A/N*(1+SMOOTH_PAR).
% 
%   [BETA,K] = GRL_ALGORITHM(X,Y,BLOCKS,TAU,SMOOTH_PAR) returns also  the number of iterations.
%
%   [...] = GRL_ALGORITHM(X,Y,BLOCKS,TAU,SMOOTH_PAR,BETA0) uses BETA0 as
%   initialization of the iterative algorithm. If BETA0=[], sets BETA0=0
%
%   [...] = GRL_ALGORITHM(X,Y,BLOCKS,TAU,SMOOTH_PAR,BETA0,SIGMA0) sets the smoothness
%   parameter to SIGMA0*SMOOTH_PAR and and the step size to SIGMA0*(1+SMOOTH_PAR). 
%   If SIGMA0=[], sets the smoothness parameter to A/N*SMOOTH_PAR and the 
%   step size is A/N*(1+SMOOTH_PAR).
%
%   [...] = GRL_ALGORITHM(X,Y,BLOCKS,TAU,SMOOTH_PAR,BETA0,SIGMA0,MAX_ITER) the algorithm stops after
%   MAX_ITER iterations or when regularized empirical error reaches convergence 
%   (default tolerance is 1e-6). If MAX_ITER=[], sets MAX_ITER=1e5.
%
%   [...] = GRL_ALGORITHM(X,Y,BLOCKS,TAU,SMOOTH_PAR,BETA0,SIGMA0,MAX_ITER,TOL) uses TOL
%   as tolerance for stopping. 


   
    if nargin<4; error('too few inputs!'); end
    if nargin<5; smooth_par = 0; end
    if nargin<6; beta0 = []; end
    if nargin<7; sigma0=[]; end
    if nargin<8; max_iter = 1e5; end   
    if nargin<9; tol = 1e-6; end
    if nargin>9; error('too many inputs!'); end

    [n,d] =size(X);
    XT = X';

    % if only the number of blocks is given, build blocks of equal 
    % cardinality with sequential features    
    if ~iscell(blocks);
        blocks = num2cell(reshape(1:d,d/blocks,blocks),1);
    end

    % if sigma is not specified in input, evaluates it as a/n
    if isempty(sigma0);
        sigma0 = normest(X*X')/n; %step size for smooth_par=0
    end
    
    mu = smooth_par*sigma0; % smoothness parameter is rescaled
    
    sigma = sigma0+mu; % step size

    % useful quantities
    mu_s = mu/sigma;
    tau_s = tau/sigma;    % twice the threshold
    XT = XT./(n*sigma);

    % initialization 
    if isempty(beta0); % initialize with RLS solution
        beta0 = zeros(d,1); 
    end
    n_iter=0;
    beta = beta0;
    h = beta0;
    t = .5;
    c = 1;
    stop = 0;
    % grl iterations

    while and(n_iter<max_iter,~stop)
        
        beta_prev = beta;   
        tp = t;
        
        t = (-c +sqrt(c^2+8*c))/4;
        fact1 = (1-t+t/tp);
        fact2 = ((tp-1)*(t/tp));
        
        beta_noproj = h.*(1-mu_s) + XT*(Y-X*h);
        beta = cell(length(blocks),1);
        for j=1:length(blocks);
            norm_beta = norm(beta_noproj(blocks{j}));
            beta{j} = max(1-(tau_s./norm_beta),0).*beta_noproj(blocks{j});
        end
        beta = cell2mat(beta);
   
        h = beta.*fact1 + beta_prev.*fact2;

        stop = norm(beta-beta_prev)<=(norm(beta_prev)*tol);
        n_iter = n_iter+1;
    end
end