function [selected,sparsity,k,beta] = grl_regpath(X,Y,blocks,tau_values,varargin)
% GRL_REGPATH acceleration of grl through cascade of grl w. decreasing 
% values of TAU_VALUES
% 
% [SELECTED] = GRL_REGPATH(X,Y,BLOCKS,TAU_VALUES) for each value in TAU_VALUES
%   evaluates grl solution with l2 parameter 0, and builds cell of indexes 
%   of selected features. X and Y are the input matrices and output vectors, respectively, 
%   BLOCKS is either the number of blocks (with equal cardinality) or a cell array, 
%   where element i contains the indexes of the features in block i.
% 
% [SELECTED,SPARSITY] = GRL_REGPATH(X,Y,BLOCKS,TAU_VALUES) also returns a vector
%   with the number of selected features for each value in TAU_VALUES
% 
% [SELECTED,SPARSITY,K] = GRL_REGPATH(X,Y,BLOCKS,TAU_VALUES) also returns a 
%   vector with the number of kations for each value in TAU_VALUES
% 
% [SELECTED,SPARSITY,K,BETA] = GRL_REGPATH(X,Y,TAU_VALUES) also returns 
%  the coefficients vector for each value in TAU_VALUES
% 
% GRL_REGPATH(...,'PropertyName',PropertyValue,...) sets properties to the
%   specified property values.
%       -'mu_fact': (default is 0) sets l2 parameter equal to MU_FACT*step_size
%       -'max_iter': (default is 1e5) maximum number of iterations
%       -'tolerance': (default is 1e-6) tolerance for stopping the iterations.
%       -'all_path': (default is true) if true evaluates all regularization
%        path with given tolerance. Otherwise evaluates solutions for larger 
%        vaues of tau with looser tolerance
   
    if nargin<4; error('too few inputs!'); end
    
    % DEFAULT PARAMETERS
    mu_fact = 0;
    kmax = 1e5;
    tol = 1e-6;
    all_path = true;

    % OPTIONAL PARAMETERS
    args = varargin;
    nargs = length(args);
    for i=1:2:nargs
        switch args{i},
            case 'mu_fact'
                mu_fact = args{i+1};
            case 'max_iter'
                kmax = args{i+1};
            case 'tolerance'
                tol = args{i+1};
            case 'all_path'
                all_path = args{i+1};
        end
    end

    T = length(tau_values);
    [n,d] = size(X);
    
    % if interested in just the first value of TAU_VALUES, evaluates solutions
    % for larger vaues of tau with looser tolerance
    tol = ones(T,1).*tol;
    if ~all_path;
        tol(2:end) = tol(2:end).*100;
    end
    
    if n<=d;
        sigma = normest(X*X')/n; %step size
    else
        sigma = normest(X)^2/n; %step size
    end

    beta = cell(T,1);
    selected = zeros(d,T);
    k = zeros(T,1);
    sparsity = zeros(T,1);
    beta0 = zeros(d,1);
    sparsity_prev = 0;

    for t = 1:T;
        % when mu=0, if for larger value of tau l1l2 selected less than n 
        % variables, then keep running l1l2 for smaller values, 
        % o.w. take rls solution
        if sparsity_prev>=n;
            if ~exist('beta_ls','var');
                if nargout==4;
                    beta_ls =rls_algorithm(X,Y);
                else
                    beta_ls = ones(d,1);
                end
            end
            beta{T+1-t} = beta_ls;
        else
            [beta{T+1-t},k(T+1-t)] = grl_algorithm(X,Y,blocks,tau_values(T+1-t),mu_fact,beta0,sigma,kmax,tol(T+1-t));
        end
        beta0 = beta{T+1-t};
        selected(:,T+1-t) = beta{T+1-t}~=0; % selected variables
        sparsity(T+1-t) = sum(selected(:,T+1-t)); % number of selected variables
        sparsity_prev = sparsity(T+1-t);
    end
   
    if ~all_path;
        beta = beta{1};
        selected = selected(:,1);
        sparsity = sparsity(1);
    end 
end